<h1>{{header}}</h1>
{{#hasItems}}
  <ul>
    {{#items}}
      {{#current}}
        <li><strong>{{name}}</strong></li>
      {{/current}}
      {{^current}}
        <li><a href="{{url}}">{{name}}</a></li>
      {{/current}}
    {{/items}}
  </ul>
{{/hasItems}}
